/**
 * WEBLAB: Service oriented integration platform for media mining and intelligence applications
 * 
 * Copyright (C) 2004 - 2011 Cassidian, an EADS company
 * 
 * This library is free software; you can redistribute it and/or modify it under the terms of
 * the GNU Lesser General Public License as published by the Free Software Foundation; either
 * version 2.1 of the License, or (at your option) any later version.
 * 
 * This library is distributed in the hope that it will be useful, but WITHOUT ANY WARRANTY;
 * without even the implied warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.
 * See the GNU Lesser General Public License for more details.
 * 
 * You should have received a copy of the GNU Lesser General Public License along with this
 * library; if not, write to the Free Software Foundation, Inc., 51 Franklin Street, Fifth
 * Floor, Boston, MA 02110-1301 USA
 */
package org.ow2.weblab.tools.maven.util;

import java.util.Arrays;
import java.util.Collections;
import java.util.HashSet;
import java.util.Set;

public final class Convention {


	private static final String REGEX_WARN_PACKAGE = "([a-z]([a-zA-Z0-9])*)(\\.[a-z]([a-zA-Z0-9])*)*";


	private static final String REGEX_ERROR_PACKAGE = "([A-Z]([a-zA-Z0-9])*)(\\.[a-zA-Z]([a-zA-Z0-9])*)*";


	private static final String REGEX_WARN_CLASSNAME = "([A-Z]([a-zA-Z0-9])*)";


	private static final String REGEX_ERROR_CLASSNAME = "([a-zA-Z]([a-zA-Z0-9])*)";


	protected static final Set<String> PROHIBITED_SET = Collections.unmodifiableSet(new HashSet<String>(Arrays.asList("abstract", "assert", "boolean", "break",
			"byte", "case", "catch", "char", "class", "const", "default", "do", "double", "else", "enum", "extends", "for", "final", "finally", "goto", "if",
			"implements", "import", "instanceof", "int", "interface", "long", "native", "new", "package", "private", "protected", "public", "return", "short",
			"static", "strictfp", "super", "switch", "synchronized", "this", "throw", "throws", "transient", "try", "void", "volatile", "while")));


	/**
	 * Do not instantiate this class !
	 */
	private Convention() {
		// Do not instantiate this class !
	}


	/**
	 * Checks if package name is valid, discouraged or invalid
	 */
	public static Message checkPackage(final String packageName) {
		Message message = Message.OK;
		if (packageName == null) {
			return null;
		}
		if (Convention.prohibited(packageName)) {
			return Message.ERROR;
		}
		if (!packageName.matches(Convention.REGEX_WARN_PACKAGE)) {
			if (!packageName.matches(Convention.REGEX_ERROR_PACKAGE)) {
				message = Message.ERROR;
			} else {
				message = Message.WARN;
			}
		}
		return message;
	}


	/**
	 * Checks if grouId name is valid, discouraged or invalid
	 */
	public static Message checkGroupID(final String groupId) {
		return Convention.checkPackage(groupId);
	}


	/**
	 * Checks if class name is valid, discouraged or invalid
	 */
	public static Message checkClassName(final String className) {
		Message message = Message.OK;
		if (className == null) {
			return null;
		}
		if (Convention.prohibited(className)) {
			return Message.ERROR;
		}
		if (!className.matches(Convention.REGEX_WARN_CLASSNAME)) {
			if (!className.matches(Convention.REGEX_ERROR_CLASSNAME)) {
				message = Message.ERROR;
			} else {
				message = Message.WARN;
			}
		}
		return message;
	}


	private static boolean prohibited(final String name) {
		for (final String s : name.split("\\.")) {
			if (Convention.PROHIBITED_SET.contains(s)) {
				return true;
			}
		}
		return false;
	}
}
